@@ -1,12 +1,10 @@
-/***************************************************************************
- *   Copyright (C) 2003-2007 by Joachim Eibl <joachim.eibl at gmx.de>      *
- *   Copyright (C) 2018 Michael Reeves reeves.87@gmail.com                 *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- ***************************************************************************/
+/*
+ * KDiff3 - Text Diff And Merge Tool
+ *
+ * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
+ * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
+ * SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 /* Features of class SourceData:
 - Read a file (from the given URL) or accept data via a string.
@@ -18,7 +16,7 @@
 
 Order of operation:
  1. If data was given via a string then save it to a temp file. (see setData())
- 2. If the specified file is nonlocal (URL) copy it to a temp file.
+ 2. If the specified file is nonlocal (URL) copy it to a temp file. (TODO revisit this)
  3. If a preprocessor was specified, run the input file through it.
  4. Read the output of the preprocessor.
  5. If Uppercase was specified: Turn the read data to uppercase.
@@ -32,10 +30,14 @@ Optimizations: Skip unneeded steps.
 */
 #include "SourceData.h"
 
-#include "Utils.h"
+#include "CommentParser.h"
 #include "diff.h"
+#include "kdiff3.h"
+#include "Logging.h"
+#include "Utils.h"
 
 #include <QProcess>
+#include <QScopedPointer>
 #include <QString>
 #include <QTemporaryFile>
 #include <QTextCodec>
@@ -44,19 +46,16 @@ Optimizations: Skip unneeded steps.
 
 #include <KLocalizedString>
 
-SourceData::SourceData()
+void SourceData::setupConnections()
 {
-    m_pOptions = nullptr;
-    reset();
-}
-
-SourceData::~SourceData()
-{
-    reset();
+#ifndef AUTOTEST
+    connections.push_back(KDiff3App::encodingChanged.connect(boost::bind(&SourceData::setEncoding, this, boost::placeholders::_1)));
+#endif
 }
 
 void SourceData::reset()
 {
+    mFromClipBoard = false;
     m_pEncoding = nullptr;
     m_fileAccess = FileAccess();
     m_normalData.reset();
@@ -83,32 +82,32 @@ void SourceData::setFilename(const QString& filename)
     }
 }
 
-bool SourceData::isEmpty()
+bool SourceData::isEmpty() const
 {
     return getFilename().isEmpty();
 }
 
-bool SourceData::hasData()
+bool SourceData::hasData() const
 {
     return m_normalData.m_pBuf != nullptr;
 }
 
-bool SourceData::isValid()
+bool SourceData::isValid() const
 {
     return isEmpty() || hasData();
 }
 
-void SourceData::setOptions(Options* pOptions)
+void SourceData::setOptions(const QSharedPointer<Options> &pOptions)
 {
     m_pOptions = pOptions;
 }
 
-QString SourceData::getFilename()
+QString SourceData::getFilename() const
 {
     return m_fileAccess.absoluteFilePath();
 }
 
-QString SourceData::getAliasName()
+QString SourceData::getAliasName() const
 {
     return m_aliasName.isEmpty() ? m_fileAccess.prettyAbsPath() : m_aliasName;
 }
@@ -120,6 +119,8 @@ void SourceData::setAliasName(const QString& name)
 
 void SourceData::setFileAccess(const FileAccess& fileAccess)
 {
+    mFromClipBoard = false;
+
     m_fileAccess = fileAccess;
     m_aliasName = QString();
     if(!m_tempInputFileName.isEmpty())
@@ -127,7 +128,7 @@ void SourceData::setFileAccess(const FileAccess& fileAccess)
         m_tempFile.remove();
         m_tempInputFileName = "";
     }
-    
+
     mErrors.clear();
 }
 void SourceData::setEncoding(QTextCodec* pEncoding)
@@ -135,7 +136,7 @@ void SourceData::setEncoding(QTextCodec* pEncoding)
     m_pEncoding = pEncoding;
 }
 
-QStringList SourceData::setData(const QString& data)
+const QString SourceData::setData(const QString& data)
 {
     mErrors.clear();
     // Create a temp file for preprocessing:
@@ -144,44 +145,45 @@ QStringList SourceData::setData(const QString& data)
         FileAccess::createTempFile(m_tempFile);
         m_tempInputFileName = m_tempFile.fileName();
     }
-
-    FileAccess f(m_tempInputFileName);
+    m_fileAccess = FileAccess(m_tempInputFileName);
     QByteArray ba = QTextCodec::codecForName("UTF-8")->fromUnicode(data);
-    bool bSuccess = f.writeFile(ba.constData(), ba.length());
+    bool bSuccess = m_fileAccess.writeFile(ba.constData(), ba.length());
     if(!bSuccess)
     {
         mErrors.append(i18n("Writing clipboard data to temp file failed."));
+        return mErrors[0];
     }
     else
     {
         m_aliasName = i18n("From Clipboard");
-        m_fileAccess = FileAccess(""); // Effect: m_fileAccess.isValid() is false
+        mFromClipBoard = true;
+        //m_fileAccess = FileAccess(); // Insure m_fileAccess is not valid
     }
 
-    return mErrors;
+    return QLatin1String("");
 }
 
-const LineData* SourceData::getLineDataForDiff() const
+const QVector<LineData>* SourceData::getLineDataForDiff() const
 {
     if(m_lmppData.m_pBuf == nullptr)
-        return m_normalData.m_v.size() > 0 ? &m_normalData.m_v[0] : nullptr;
+        return m_normalData.m_v.size() > 0 ? &m_normalData.m_v : nullptr;
     else
-        return m_lmppData.m_v.size() > 0 ? &m_lmppData.m_v[0] : nullptr;
+        return m_lmppData.m_v.size() > 0 ? &m_lmppData.m_v : nullptr;
 }
 
-const LineData* SourceData::getLineDataForDisplay() const
+const QVector<LineData>* SourceData::getLineDataForDisplay() const
 {
-    return m_normalData.m_v.size() > 0 ? &m_normalData.m_v[0] : nullptr;
+    return m_normalData.m_v.size() > 0 ? &m_normalData.m_v : nullptr;
 }
 
 LineRef SourceData::getSizeLines() const
 {
-    return (LineRef)m_normalData.m_vSize;
+    return m_normalData.lineCount();
 }
 
 qint64 SourceData::getSizeBytes() const
 {
-    return m_normalData.m_size;
+    return m_normalData.byteCount();
 }
 
 const char* SourceData::getBuf() const
@@ -191,31 +193,36 @@ const char* SourceData::getBuf() const
 
 const QString& SourceData::getText() const
 {
-    return m_normalData.m_unicodeBuf;
+    return *m_normalData.m_unicodeBuf;
 }
 
-bool SourceData::isText()
+bool SourceData::isText() const
 {
     return m_normalData.isText() || m_normalData.isEmpty();
 }
 
-bool SourceData::isIncompleteConversion()
+bool SourceData::isIncompleteConversion() const
 {
     return m_normalData.m_bIncompleteConversion;
 }
 
-bool SourceData::isFromBuffer()
+bool SourceData::isFromBuffer() const
 {
-    return !m_fileAccess.isValid();
+    return mFromClipBoard;
 }
 
-bool SourceData::isBinaryEqualWith(const SourceData& other) const
+bool SourceData::isBinaryEqualWith(const QSharedPointer<SourceData>& other) const
 {
-    return m_fileAccess.exists() && other.m_fileAccess.exists() &&
-           getSizeBytes() == other.getSizeBytes() &&
-           (getSizeBytes() == 0 || memcmp(getBuf(), other.getBuf(), getSizeBytes()) == 0);
+    return m_fileAccess.exists() && other->m_fileAccess.exists() &&
+           getSizeBytes() == other->getSizeBytes() &&
+           (getSizeBytes() == 0 || memcmp(getBuf(), other->getBuf(), getSizeBytes()) == 0);
 }
 
+/*
+    Warning: Do not call this function without re-running the comparison or
+    otherwise resetting the DiffTextWindows as these store a pointer to the file
+    data stored here.
+*/
 void SourceData::FileData::reset()
 {
     if(m_pBuf != nullptr)
@@ -224,8 +231,8 @@ void SourceData::FileData::reset()
         m_pBuf = nullptr;
     }
     m_v.clear();
-    m_size = 0;
-    m_vSize = 0;
+    mDataSize = 0;
+    mLineCount = 0;
     m_bIsText = false;
     m_bIncompleteConversion = false;
     m_eLineEndStyle = eLineEndStyleUndefined;
@@ -234,7 +241,8 @@ void SourceData::FileData::reset()
 bool SourceData::FileData::readFile(FileAccess& file)
 {
     reset();
-    if(file.fileName().isEmpty()) {
+    if(file.fileName().isEmpty())
+    {
         return true;
     }
 
@@ -243,25 +251,25 @@ bool SourceData::FileData::readFile(FileAccess& file)
     if(!file.isNormal())
         return true;
 
-    m_size = file.sizeForReading();
+    mDataSize = file.sizeForReading();
     char* pBuf;
-    m_pBuf = pBuf = new char[m_size + 100]; // Alloc 100 byte extra: Safety hack, not nice but does no harm.
-                                            // Some extra bytes at the end of the buffer are needed by
-                                            // the diff algorithm. See also GnuDiff::diff_2_files().
-    bool bSuccess = file.readFile(pBuf, m_size);
+    m_pBuf = pBuf = new char[mDataSize + 100]; // Alloc 100 byte extra: Safety hack, not nice but does no harm.
+                                               // Some extra bytes at the end of the buffer are needed by
+                                               // the diff algorithm. See also GnuDiff::diff_2_files().
+    bool bSuccess = file.readFile(pBuf, mDataSize);
     if(!bSuccess)
     {
         delete[] pBuf;
         m_pBuf = nullptr;
-        m_size = 0;
+        mDataSize = 0;
     }
     else
     {
         //null terminate buffer
-        pBuf[m_size + 1] = 0;
-        pBuf[m_size + 2] = 0;
-        pBuf[m_size + 3] = 0;
-        pBuf[m_size + 4] = 0;
+        pBuf[mDataSize + 1] = 0;
+        pBuf[mDataSize + 2] = 0;
+        pBuf[mDataSize + 3] = 0;
+        pBuf[mDataSize + 4] = 0;
     }
     return bSuccess;
 }
@@ -269,7 +277,8 @@ bool SourceData::FileData::readFile(FileAccess& file)
 bool SourceData::FileData::readFile(const QString& filename)
 {
     reset();
-    if(filename.isEmpty()) {
+    if(filename.isEmpty())
+    {
         return true;
     }
 
@@ -278,17 +287,17 @@ bool SourceData::FileData::readFile(const QString& filename)
     if(!fa.isNormal())
         return true;
 
-    m_size = fa.sizeForReading();
+    mDataSize = fa.sizeForReading();
     char* pBuf;
-    m_pBuf = pBuf = new char[m_size + 100]; // Alloc 100 byte extra: Safety hack, not nice but does no harm.
-                                            // Some extra bytes at the end of the buffer are needed by
-                                            // the diff algorithm. See also GnuDiff::diff_2_files().
-    bool bSuccess = fa.readFile(pBuf, m_size);
+    m_pBuf = pBuf = new char[mDataSize + 100]; // Alloc 100 byte extra: Safety hack, not nice but does no harm.
+                                               // Some extra bytes at the end of the buffer are needed by
+                                               // the diff algorithm. See also GnuDiff::diff_2_files().
+    bool bSuccess = fa.readFile(pBuf, mDataSize);
     if(!bSuccess)
     {
         delete[] pBuf;
         m_pBuf = nullptr;
-        m_size = 0;
+        mDataSize = 0;
     }
     return bSuccess;
 }
@@ -300,23 +309,25 @@ bool SourceData::saveNormalDataAs(const QString& fileName)
 
 bool SourceData::FileData::writeFile(const QString& filename)
 {
-    if(filename.isEmpty()) {
+    if(filename.isEmpty())
+    {
         return true;
     }
 
     FileAccess fa(filename);
-    bool bSuccess = fa.writeFile(m_pBuf, m_size);
+    bool bSuccess = fa.writeFile(m_pBuf, mDataSize);
     return bSuccess;
 }
 
-void SourceData::FileData::copyBufFrom(const FileData& src)
+//Depriated
+void SourceData::FileData::copyBufFrom(const FileData& src) //TODO: Remove me.
 {
     reset();
     char* pBuf;
-    m_size = src.m_size;
-    m_pBuf = pBuf = new char[m_size + 100];
+    mDataSize = src.mDataSize;
+    m_pBuf = pBuf = new char[mDataSize + 100];
     Q_ASSERT(src.m_pBuf != nullptr);
-    memcpy(pBuf, src.m_pBuf, m_size);
+    memcpy(pBuf, src.m_pBuf, mDataSize);
 }
 
 QTextCodec* SourceData::detectEncoding(const QString& fileName, QTextCodec* pFallbackCodec)
@@ -334,7 +345,7 @@ QTextCodec* SourceData::detectEncoding(const QString& fileName, QTextCodec* pFal
     return pFallbackCodec;
 }
 
-const QStringList& SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicode)
+void SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicode)
 {
     m_pEncoding = pEncoding;
     QTemporaryFile fileIn1, fileOut1;
@@ -343,18 +354,19 @@ const QStringList& SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAu
     QString fileNameIn2;
     QString fileNameOut2;
 
-    if(m_fileAccess.isValid() && !m_fileAccess.isNormal())
-    {
-        mErrors.append(i18n("%1 is not a normal file.", m_fileAccess.prettyAbsPath()));
-        return mErrors;
-    }
-
-    Q_ASSERT(!m_fileAccess.isDir());
-    bool bTempFileFromClipboard = !m_fileAccess.isValid();
-
     // Detect the input for the preprocessing operations
-    if(!bTempFileFromClipboard)
+    if(!mFromClipBoard)
     {
+        //Routine result of directory compare finding a file that isn't in all locations.
+        if(!m_fileAccess.isValid()) return;
+
+        Q_ASSERT(!m_fileAccess.isDir());
+        if(!m_fileAccess.isNormal())
+        {
+            mErrors.append(i18n("%1 is not a normal file.", m_fileAccess.prettyAbsPath()));
+            return;
+        }
+
         if(m_fileAccess.isLocal())
         {
             fileNameIn1 = m_fileAccess.absoluteFilePath();
@@ -376,6 +388,12 @@ const QStringList& SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAu
     }
     else // The input was set via setData(), probably from clipboard.
     {
+        /*
+            Happens during initial startup. This means we have nothing to compare
+        */
+        if(m_tempInputFileName.isEmpty())
+            return;
+
         fileNameIn1 = m_tempInputFileName;
         m_pEncoding = QTextCodec::codecForName("UTF-8");
     }
@@ -397,7 +415,7 @@ const QStringList& SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAu
             if(!m_normalData.readFile(faIn))
             {
                 mErrors.append(faIn.getStatusText());
-                return mErrors;
+                return;
             }
         }
         else
@@ -433,15 +451,15 @@ const QStringList& SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAu
                 errorReason = "\n(" + errorReason + ')';
 
             bool bSuccess = errorReason.isEmpty() && m_normalData.readFile(fileNameOut1);
-            if(fileInSize > 0 && (!bSuccess || m_normalData.m_size == 0))
+            if(fileInSize > 0 && (!bSuccess || m_normalData.byteCount() == 0))
             {
+                //Don't fail the preprocessor command if the file can't be read.
                 if(!m_normalData.readFile(faIn))
                 {
                     mErrors.append(faIn.getStatusText());
                     mErrors.append(i18n("    Temp file is: %1", fileNameIn1));
-                    return mErrors;
+                    return;
                 }
-                //Don't fail the preprocessor command if the file cann't be read.
                 mErrors.append(
                     i18n("Preprocessing possibly failed. Check this command:\n\n  %1"
                          "\n\nThe preprocessing command will be disabled now.", ppCmd) +
@@ -452,14 +470,14 @@ const QStringList& SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAu
             }
         }
 
-        if(!m_normalData.preprocess(m_pOptions->m_bPreserveCarriageReturn, pEncoding1))
+        if(!m_normalData.preprocess(pEncoding1, false))
         {
             mErrors.append(i18n("File %1 too large to process. Skipping.", fileNameIn1));
-            return mErrors;
+            return;
         }
         //exit early for non text data further processing assumes a text file as input
         if(!m_normalData.isText())
-            return mErrors;
+            return;
 
         // LineMatching Preprocessor
         if(!m_pOptions->m_LineMatchingPreProcessorCmd.isEmpty())
@@ -495,7 +513,7 @@ const QStringList& SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAu
                 errorReason = "\n(" + errorReason + ')';
 
             bool bSuccess = errorReason.isEmpty() && m_lmppData.readFile(fileNameOut2);
-            if(FileAccess(fileNameIn2).size() > 0 && (!bSuccess || m_lmppData.m_size == 0))
+            if(FileAccess(fileNameIn2).size() > 0 && (!bSuccess || m_lmppData.byteCount() == 0))
             {
                 mErrors.append(
                     i18n("The line-matching-preprocessing possibly failed. Check this command:\n\n  %1"
@@ -505,11 +523,11 @@ const QStringList& SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAu
                 if(!m_lmppData.readFile(fileNameIn2))
                 {
                     mErrors.append(i18n("Failed to read file: %1", fileNameIn2));
-                    return mErrors;
+                    return;
                 }
             }
         }
-        else if(m_pOptions->m_bIgnoreComments || m_pOptions->m_bIgnoreCase)
+        else if(m_pOptions->ignoreComments() || m_pOptions->m_bIgnoreCase)
         {
             // We need a copy of the normal data.
             m_lmppData.copyBufFrom(m_normalData);
@@ -517,328 +535,176 @@ const QStringList& SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAu
     }
     else
     {
-        //exit early for non-existant files
-        return mErrors;
+        //exit early for nonexistent files
+        return;
     }
 
-    if(!m_lmppData.preprocess(false, pEncoding2))
+    if(!m_lmppData.preprocess(pEncoding2, true))
     {
         mErrors.append(i18n("File %1 too large to process. Skipping.", fileNameIn1));
-        return mErrors;
+        return;
     }
 
     Q_ASSERT(m_lmppData.isText());
     //TODO: Needed?
-    if(m_lmppData.m_vSize < m_normalData.m_vSize)
+    if(m_lmppData.lineCount() < m_normalData.lineCount())
     {
         // Preprocessing command may result in smaller data buffer so adjust size
-        m_lmppData.m_v.resize((int)m_normalData.m_vSize);
-        for(qint64 i = m_lmppData.m_vSize; i < m_normalData.m_vSize; ++i)
+        for(qint64 i = m_lmppData.lineCount(); i < m_normalData.lineCount(); ++i)
         { // Set all empty lines to point to the end of the buffer.
-            m_lmppData.m_v[(int)i].setLine(m_lmppData.m_unicodeBuf.unicode() + m_lmppData.m_unicodeBuf.length());
+            m_lmppData.m_v.push_back(LineData(m_lmppData.m_unicodeBuf, m_lmppData.m_unicodeBuf->length()));
         }
 
-        m_lmppData.m_vSize = m_normalData.m_vSize;
+        m_lmppData.mLineCount = m_normalData.lineCount();
     }
 
     // Ignore comments
-    if(m_pOptions->m_bIgnoreComments)
+    if(m_pOptions->ignoreComments() && hasData())
     {
-        m_lmppData.removeComments();
-        LineRef vSize = (LineRef)std::min(m_normalData.m_vSize, m_lmppData.m_vSize);
-        for(int i = 0; i < (int)vSize; ++i)
+        qint64 vSize = std::min(m_normalData.lineCount(), m_lmppData.lineCount());
+        Q_ASSERT(vSize < TYPE_MAX(qint32));
+        //Perform explcit cast to insure well defined behavior comparing 32-bit to a 64-bit value
+        for(qint32 i = 0; (qint64)i < vSize; ++i)
         {
-            m_normalData.m_v[i].setPureComment(m_lmppData.m_v[i].isPureComment());
+            //TODO: Phase this out. We should not be messing with these flags outside the parser.
+            m_normalData.m_v[i].setSkipable(m_lmppData.m_v[i].isPureComment());
+            m_normalData.m_v[i].setSkipable(m_lmppData.m_v[i].isSkipable());
         }
     }
-
-    return mErrors;
 }
 
 /** Prepare the linedata vector for every input line.*/
-bool SourceData::FileData::preprocess(bool bPreserveCR, QTextCodec* pEncoding)
+bool SourceData::FileData::preprocess(QTextCodec* pEncoding, bool removeComments)
 {
-    qint64 i;
+    if(m_pBuf == nullptr)
+        return true;
+
+    if(pEncoding == nullptr)
+        return false;
+
+    QString line;
+    QChar curChar;
+    LineCount lineCount = 0;
+    qint64 lastOffset = 0;
+    qint64 skipBytes = 0;
+    QScopedPointer<CommentParser> parser(new DefaultCommentParser());
+
     // detect line end style
     QVector<e_LineEndStyle> vOrigDataLineEndStyle;
     m_eLineEndStyle = eLineEndStyleUndefined;
-    for(i = 0; i < m_size; ++i)
+
+    QTextCodec* pCodec = detectEncoding(m_pBuf, mDataSize, skipBytes);
+    if(pCodec != pEncoding)
+        skipBytes = 0;
+
+    if(mDataSize - skipBytes > TYPE_MAX(QtNumberType))
+        return false;
+
+    const QByteArray ba = QByteArray::fromRawData(m_pBuf + skipBytes, (int)(mDataSize - skipBytes));
+    QTextStream ts(ba, QIODevice::ReadOnly); //Don't use text mode we need to see the actual line ending.
+    ts.setCodec(pEncoding);
+    ts.setAutoDetectUnicode(false);
+
+    m_bIncompleteConversion = false;
+    m_unicodeBuf->clear();
+    Q_ASSERT(m_unicodeBuf->length() == 0);
+
+    while(!ts.atEnd())
     {
-        if(m_pBuf[i] == '\r')
+        line.clear();
+        if(lineCount >= TYPE_MAX(LineCount) - 5)
         {
-            if(i + 1 < m_size && m_pBuf[i + 1] == '\n') // not 16-bit unicode
+            m_v.clear();
+            return false;
+        }
+
+        ts >> curChar;
+
+        quint32 firstNonwhite = 0;
+        bool    foundNonWhite = false;
+
+        //QTextStream::readLine doesn't tell us about line endings.
+        while(curChar != '\n' && curChar != '\r')
+        {
+            if(curChar.isNull() || curChar.isNonCharacter())
             {
-                vOrigDataLineEndStyle.push_back(eLineEndStyleDos);
-                ++i;
+                m_v.clear();
+                return true;
             }
-            else if(i > 0 && i + 2 < m_size && m_pBuf[i - 1] == '\0' && m_pBuf[i + 1] == '\0' && m_pBuf[i + 2] == '\n') // 16-bit unicode
+
+            if(curChar == QChar::ReplacementCharacter)
+                m_bIncompleteConversion = true;
+
+            line.append(curChar);
+            if(!curChar.isSpace() && !foundNonWhite)
             {
-                vOrigDataLineEndStyle.push_back(eLineEndStyleDos);
-                i += 2;
+                firstNonwhite = line.length();
+                foundNonWhite = true;
             }
-            else // old mac line end style ?
-            {
+
+            if(ts.atEnd())
+                break;
+
+            ts >> curChar;
+        }
+
+        ++lineCount;
+
+        switch(curChar.unicode())
+        {
+            case '\n':
+                vOrigDataLineEndStyle.push_back(eLineEndStyleUnix);
+                break;
+            case '\r':
+                if(lastOffset < mDataSize)
+                {
+                    //workaround for lack of peak API in QTextStream.
+                    qint64 j;
+                    for(j = 0; j < 4 && lastOffset + j < mDataSize; ++j)
+                    {
+                        if(m_pBuf[lastOffset + j] != '\0')
+                            break;
+                    }
+
+                    if(m_pBuf[lastOffset + j] == '\n')
+                    {
+                        ts >> curChar;
+                        vOrigDataLineEndStyle.push_back(eLineEndStyleDos);
+                        lastOffset = ts.pos();
+                        break;
+                    }
+                }
+
+                //old mac style ending.
                 vOrigDataLineEndStyle.push_back(eLineEndStyleUndefined);
-                const_cast<char*>(m_pBuf)[i] = '\n'; // fix it in original data
-            }
-        }
-        else if(m_pBuf[i] == '\n')
-        {
-            vOrigDataLineEndStyle.push_back(eLineEndStyleUnix);
+                break;
         }
+        parser->processLine(line);
+        if(removeComments)
+            parser->removeComment(line);
+
+        //kdiff3 internally uses only unix style endings for simplicity.
+        m_v.push_back(LineData(m_unicodeBuf, lastOffset, line.length(), firstNonwhite, parser->isSkipable(), parser->isPureComment()));
+        m_unicodeBuf->append(line).append('\n');
+
+        lastOffset = m_unicodeBuf->length();
     }
 
+    m_v.push_back(LineData(m_unicodeBuf, lastOffset));
+    Q_ASSERT(m_v.size() < 2 || m_v[m_v.size() - 1].getOffset() != m_v[m_v.size() - 2].getOffset());
+
+    m_bIsText = true;
+
     if(!vOrigDataLineEndStyle.isEmpty())
         m_eLineEndStyle = vOrigDataLineEndStyle[0];
 
-    qint64 skipBytes = 0;
-    QTextCodec* pCodec = detectEncoding(m_pBuf, m_size, skipBytes);
-    if(pCodec != pEncoding)
-        skipBytes = 0;
-
-    if(m_size - skipBytes > INT_MAX)
-        return false;
-
-    QByteArray ba = QByteArray::fromRawData(m_pBuf + skipBytes, (int)(m_size - skipBytes));
-    QTextStream ts(ba, QIODevice::ReadOnly | QIODevice::Text);
-    ts.setCodec(pEncoding);
-    ts.setAutoDetectUnicode(false);
-    m_unicodeBuf = ts.readAll();
-    ba.clear();
-
-    int ucSize = m_unicodeBuf.length();
-    const QChar* p = m_unicodeBuf.unicode();
-
-    m_bIsText = true;
-    int lines = 1;
-    m_bIncompleteConversion = false;
-    for(i = 0; i < ucSize; ++i)
-    {
-        if(i >= ucSize || p[i] == '\n')
-        {
-            if(lines >= INT_MAX - 5)
-                return false;
-
-            ++lines;
-        }
-        if(p[i].isNull())
-        {
-            m_bIsText = false;
-        }
-        if(p[i] == QChar::ReplacementCharacter)
-        {
-            m_bIncompleteConversion = true;
-        }
-    }
-
-    m_v.resize(lines + 5);
-    int lineIdx = 0;
-    int lineLength = 0;
-    bool bNonWhiteFound = false;
-    int whiteLength = 0;
-    for(i = 0; i <= ucSize; ++i)
-    {
-        if(i >= ucSize || p[i] == '\n')
-        {
-            const QChar* pLine = &p[i - lineLength];
-            m_v[lineIdx].setLine(&p[i - lineLength]);
-            while(/*!bPreserveCR  &&*/ lineLength > 0 && m_v[lineIdx].getLine()[lineLength - 1] == '\r')
-            {
-                --lineLength;
-            }
-            m_v[lineIdx].setFirstNonWhiteChar(m_v[lineIdx].getLine() + std::min(whiteLength, lineLength));
-            if(lineIdx < vOrigDataLineEndStyle.count() && bPreserveCR && i < ucSize)
-            {
-                ++lineLength;
-                const_cast<QChar*>(pLine)[lineLength] = '\r';
-                //switch ( vOrigDataLineEndStyle[lineIdx] )
-                //{
-                //case eLineEndStyleUnix: const_cast<QChar*>(pLine)[lineLength] = '\n'; break;
-                //case eLineEndStyleDos:  const_cast<QChar*>(pLine)[lineLength] = '\r'; break;
-                //case eLineEndStyleUndefined: const_cast<QChar*>(pLine)[lineLength] = '\x0b'; break;
-                //}
-            }
-            m_v[lineIdx].setSize(lineLength);
-
-            lineLength = 0;
-            bNonWhiteFound = false;
-            whiteLength = 0;
-            ++lineIdx;
-        }
-        else
-        {
-            ++lineLength;
-
-            if(!bNonWhiteFound && isWhite(p[i]))
-                ++whiteLength;
-            else
-                bNonWhiteFound = true;
-        }
-    }
-    Q_ASSERT(lineIdx == lines);
-
-    m_vSize = lines;
+    mLineCount = lineCount;
     return true;
 }
 
-// Must not be entered, when within a comment.
-// Returns either at a newline-character p[i]=='\n' or when i==size.
-// A line that contains only comments is still "white".
-// Comments in white lines must remain, while comments in
-// non-white lines are overwritten with spaces.
-void SourceData::FileData::checkLineForComments(
-    const QChar* p,          // pointer to start of buffer
-    int& i,                  // index of current position (in, out)
-    int size,                // size of buffer
-    bool& bWhite,            // false if this line contains nonwhite characters (in, out)
-    bool& bCommentInLine,    // true if any comment is within this line (in, out)
-    bool& bStartsOpenComment // true if the line ends within an comment (out)
-    )
-{
-    bStartsOpenComment = false;
-    for(; i < size; ++i)
-    {
-        // A single apostroph ' has prio over a double apostroph " (e.g. '"')
-        // (if not in a string)
-        if(p[i] == '\'')
-        {
-            bWhite = false;
-            ++i;
-            for(; !isLineOrBufEnd(p, i, size) && p[i] != '\''; ++i)
-                ;
-            if(p[i] == '\'') ++i;
-        }
-
-        // Strings have priority over comments: e.g. "/* Not a comment, but a string. */"
-        else if(p[i] == '"')
-        {
-            bWhite = false;
-            ++i;
-            for(; !isLineOrBufEnd(p, i, size) && !(p[i] == '"' && p[i - 1] != '\\'); ++i)
-                ;
-            if(p[i] == '"') ++i;
-        }
-
-        // C++-comment
-        else if(p[i] == '/' && i + 1 < size && p[i + 1] == '/')
-        {
-            int commentStart = i;
-            bCommentInLine = true;
-            i += 2;
-            for(; !isLineOrBufEnd(p, i, size); ++i)
-                ;
-            if(!bWhite)
-            {
-                size = i - commentStart;
-                m_unicodeBuf.replace(commentStart, size, QString(" ").repeated(size));
-            }
-            return;
-        }
-
-        // C-comment
-        else if(p[i] == '/' && i + 1 < size && p[i + 1] == '*')
-        {
-            int commentStart = i;
-            bCommentInLine = true;
-            i += 2;
-            for(; !isLineOrBufEnd(p, i, size); ++i)
-            {
-                if(i + 1 < size && p[i] == '*' && p[i + 1] == '/') // end of the comment
-                {
-                    i += 2;
-
-                    // More comments in the line?
-                    checkLineForComments(p, i, size, bWhite, bCommentInLine, bStartsOpenComment);
-                    if(!bWhite)
-                    {
-                        size = i - commentStart;
-                        m_unicodeBuf.replace(commentStart, size, QString(" ").repeated(size));                    }
-                    return;
-                }
-            }
-            bStartsOpenComment = true;
-            return;
-        }
-
-        if(isLineOrBufEnd(p, i, size))
-        {
-            return;
-        }
-        else if(!p[i].isSpace())
-        {
-            bWhite = false;
-        }
-    }
-}
-
-// Modifies the input data, and replaces C/C++ comments with whitespace
-// when the line contains other data too. If the line contains only
-// a comment or white data, remember this in the flag bContainsPureComment.
-void SourceData::FileData::removeComments()
-{
-    int line = 0;
-    const QChar* p = m_unicodeBuf.unicode();
-    bool bWithinComment = false;
-    int size = m_unicodeBuf.length();
-    for(int i = 0; i < size; ++i)
-    {
-        //      std::cout << "2        " << std::string(&p[i], m_v[line].size) << std::endl;
-        bool bWhite = true;
-        bool bCommentInLine = false;
-
-        if(bWithinComment)
-        {
-            int commentStart = i;
-            bCommentInLine = true;
-
-            for(; !isLineOrBufEnd(p, i, size); ++i)
-            {
-                if(i + 1 < size && p[i] == '*' && p[i + 1] == '/') // end of the comment
-                {
-                    i += 2;
-
-                    // More comments in the line?
-                    checkLineForComments(p, i, size, bWhite, bCommentInLine, bWithinComment);
-                    if(!bWhite)
-                    {
-                        size = i - commentStart;
-                        m_unicodeBuf.replace(commentStart, size, QString(" ").repeated(size));
-                    }
-                    break;
-                }
-            }
-        }
-        else
-        {
-            checkLineForComments(p, i, size, bWhite, bCommentInLine, bWithinComment);
-        }
-
-        // end of line
-        Q_ASSERT(isLineOrBufEnd(p, i, size));
-        m_v[line].setPureComment(bCommentInLine && bWhite);
-        /*      std::cout << line << " : " <<
-       ( bCommentInLine ?  "c" : " " ) <<
-       ( bWhite ? "w " : "  ") <<
-       std::string(pLD[line].pLine, pLD[line].size) << std::endl;*/
-
-        ++line;
-    }
-}
-
-bool SourceData::isLineOrBufEnd(const QChar* p, int i, int size)
-{
-    return i >= size            // End of file
-           || isEndOfLine(p[i]) // Normal end of line
-
-        // No support for Mac-end of line yet, because incompatible with GNU-diff-routines.
-        // || ( p[i]=='\r' && (i>=size-1 || p[i+1]!='\n')
-        //                 && (i==0        || p[i-1]!='\n') )  // Special case: '\r' without '\n'
-        ;
-}
-
 // Convert the input file from input encoding to output encoding and write it to the output file.
 bool SourceData::convertFileEncoding(const QString& fileNameIn, QTextCodec* pCodecIn,
-                                const QString& fileNameOut, QTextCodec* pCodecOut)
+                                     const QString& fileNameOut, QTextCodec* pCodecOut)
 {
     QFile in(fileNameIn);
     if(!in.open(QIODevice::ReadOnly))
@@ -918,9 +784,9 @@ QTextCodec* SourceData::detectEncoding(const char* buf, qint64 size, qint64& ski
         We don't need the whole file here just the header.
 ]    */
     if(size <= 5000)
-        s=QByteArray(buf, (int)size);
+        s = QByteArray(buf, (int)size);
     else
-        s=QByteArray(buf, 5000);
+        s = QByteArray(buf, 5000);
 
     int xmlHeaderPos = s.indexOf("<?xml");
     if(xmlHeaderPos >= 0)
