@@ -1,18 +1,19 @@
-/***************************************************************************
- *   Copyright (C) 2002-2007 by Joachim Eibl <joachim.eibl at gmx.de>      *
- *   Copyright (C) 2018 Michael Reeves reeves.87@gmail.com                 *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
-
+/*
+ * KDiff3 - Text Diff And Merge Tool
+ *
+ * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
+ * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
+ * SPDX-License-Identifier: GPL-2.0-or-later
+*/
+//clazy:excludeall=connect-non-signal,incorrect-emit
 #ifndef KDIFF3_H
 #define KDIFF3_H
 
 #include "diff.h"
+#include "defmac.h"
+#include "combiners.h"
+
+#include <boost/signals2.hpp>
 
 // include files for Qt
 #include <QAction>
@@ -30,13 +31,15 @@
 #include <KToggleAction>
 // forward declaration of the KDiff3 classes
 class OptionDialog;
+
+class Overview;
+enum class e_OverviewMode;
 class FindDialog;
 //class ManualDiffHelpDialog;
 class DiffTextWindow;
 class DiffTextWindowFrame;
 class MergeResultWindow;
 class WindowTitleWidget;
-class Overview;
 
 class QStatusBar;
 class QMenu;
@@ -65,13 +68,13 @@ class ReversibleScrollBar : public QScrollBar
     {
         m_pbRightToLeftLanguage = pbRightToLeftLanguage;
         m_realVal = 0;
-        connect(this, &ReversibleScrollBar::valueChanged, this, &ReversibleScrollBar::slotValueChanged);
+        chk_connect(this, &ReversibleScrollBar::valueChanged, this, &ReversibleScrollBar::slotValueChanged);
     }
     void setAgain() { setValue(m_realVal); }
 
     void setValue(int i)
     {
-        if(m_pbRightToLeftLanguage && *m_pbRightToLeftLanguage)
+        if(m_pbRightToLeftLanguage != nullptr && *m_pbRightToLeftLanguage)
             QScrollBar::setValue(maximum() - (i - minimum()));
         else
             QScrollBar::setValue(i);
@@ -85,15 +88,31 @@ class ReversibleScrollBar : public QScrollBar
     void slotValueChanged(int i)
     {
         m_realVal = i;
-        if(m_pbRightToLeftLanguage && *m_pbRightToLeftLanguage)
+        if(m_pbRightToLeftLanguage != nullptr && *m_pbRightToLeftLanguage)
             m_realVal = maximum() - (i - minimum());
-        emit valueChanged2(m_realVal);
+        Q_EMIT valueChanged2(m_realVal);
     }
 
   Q_SIGNALS:
     void valueChanged2(int);
 };
 
+/*
+  InitFlag
+*/
+
+enum class InitFlag
+{
+    loadFiles = 1,
+    useCurrentEncoding = 2,
+    autoSolve = 4,
+    initGUI = 8,
+    defaultFlags = loadFiles | autoSolve | initGUI
+};
+
+Q_DECLARE_FLAGS(InitFlags, InitFlag);
+Q_DECLARE_OPERATORS_FOR_FLAGS(InitFlags);
+
 class KDiff3App : public QSplitter
 {
     Q_OBJECT
@@ -104,7 +123,7 @@ class KDiff3App : public QSplitter
     KDiff3App(QWidget* parent, const QString& name, KDiff3Part* pKDiff3Part);
     ~KDiff3App() override;
 
-    bool isPart();
+    bool isPart() const;
 
     /** initializes the KActions of the application */
     void initActions(KActionCollection*);
@@ -126,28 +145,60 @@ class KDiff3App : public QSplitter
      * @see KMainWindow#closeEvent
      */
     virtual bool queryClose();
-    virtual bool isFileSaved();
-    virtual bool isDirComparison();
+    virtual bool isFileSaved() const;
+    virtual bool isDirComparison() const;
+
+    static bool isTripleDiff() { return m_bTripleDiff; }
+
+    KActionCollection* actionCollection() const;
+
+    static boost::signals2::signal<QString (), FirstNonEmpty<QString>> getSelection;
+    static boost::signals2::signal<bool (), or> allowCopy;
+    static boost::signals2::signal<bool (), or> allowCut;
+    static boost::signals2::signal<bool (), and> shouldContinue;
+    static boost::signals2::signal<void (QTextCodec*)> encodingChanged;
 
   Q_SIGNALS:
     void createNewInstance(const QString& fn1, const QString& fn2, const QString& fn3);
 
-  protected:
-    void setLockPainting(bool bLock);
-    void createCaption();
-    void initDirectoryMergeActions();
-    /** sets up the statusbar for the main window by initialzing a statuslabel. */
-    void initStatusBar();
+    void sigRecalcWordWrap();
 
-    /** creates the centerwidget of the KMainWindow instance and sets it as the view */
-    void initView();
+    void finishDrop();
 
-  public Q_SLOTS:
+    void showWhiteSpaceToggled();
+    void showLineNumbersToggled();
+    void doRefresh();
+
+    void autoSolve();
+    void unsolve();
+    void mergeHistory();
+    void regExpAutoMerge();
+
+    void goCurrent();
+    void goTop();
+    void goBottom();
+    void goPrevUnsolvedConflict();
+
+    void goNextUnsolvedConflict();
+    void goPrevConflict();
+
+    void goNextConflict();
+    void goPrevDelta();
+    void goNextDelta();
+
+    void cut();
+
+    void copy();
+
+    void selectAll();
+
+    void changeOverViewMode(e_OverviewMode);
+public Q_SLOTS:
 
     /** open a file and load it into the document*/
     void slotFileOpen();
     void slotFileOpen2(QStringList &errors, const QString& fn1, const QString& fn2, const QString& fn3, const QString& ofn,
-                       const QString& an1, const QString &an2, const QString& an3, TotalDiffStatus* pTotalDiffStatus);
+                       const QString& an1, const QString& an2, const QString& an3, TotalDiffStatus* pTotalDiffStatus);
 
     void slotFileNameChanged(const QString& fileName, e_SrcSelector winIdx);
 
@@ -184,7 +235,6 @@ class KDiff3App : public QSplitter
     void slotStatusMsg(const QString& text);
 
     void resizeDiffTextWindowHeight(int newHeight);
-    void resizeMergeResultWindow();
     void slotRecalcWordWrap();
     void postRecalcWordWrap();
     void slotFinishRecalcWordWrap(int visibleTextWidth);
@@ -193,7 +243,6 @@ class KDiff3App : public QSplitter
 
     void scrollDiffTextWindow(int deltaX, int deltaY);
     void scrollMergeResultWindow(int deltaX, int deltaY);
-    void setDiff3Line(int line);
     void sourceMask(int srcMask, int enabledMask);
 
     void slotDirShowBoth();
@@ -229,7 +278,6 @@ class KDiff3App : public QSplitter
     void slotFinishMainInit();
     void slotMergeCurrentFile();
     void slotReload();
-    void slotCheckIfCanContinue(bool* pbContinue);
     void slotShowWhiteSpaceToggled();
     void slotShowLineNumbersToggled();
     void slotAutoAdvanceToggled();
@@ -249,65 +297,101 @@ class KDiff3App : public QSplitter
     void slotAddManualDiffHelp();
     void slotClearManualDiffHelpList();
     void slotNoRelevantChangesDetected();
-    void slotEncodingChangedA(QTextCodec*);
-    void slotEncodingChangedB(QTextCodec*);
-    void slotEncodingChangedC(QTextCodec*);
+    void slotEncodingChanged(QTextCodec*);
+
+    void slotFinishDrop();
+
+    void setHScrollBarRange();
+
+    void slotFocusChanged(QWidget *old, QWidget *now);
+
+  protected:
+    void setLockPainting(bool bLock);
+    void createCaption();
+    void initDirectoryMergeActions();
+    /** sets up the statusbar for the main window by initialzing a statuslabel. */
+    void initStatusBar();
+
+    /** creates the centerwidget of the KMainWindow instance and sets it as the view */
+    void initView();
+
   private:
+    void mainInit(TotalDiffStatus* pTotalDiffStatus, const InitFlags inFlags = InitFlag::defaultFlags);
+    void mainWindowEnable(bool bEnable);
+    void wheelEvent(QWheelEvent* pWheelEvent) override;
+    void keyPressEvent(QKeyEvent* event) override;
+    void resizeEvent(QResizeEvent*) override;
+
+    bool doDirectoryCompare(const bool bCreateNewInstance);
+    void improveFilenames();
+
+    bool canContinue();
+
+    void choose(e_SrcSelector choice);
+
+    QStatusBar* statusBar() const;
+    KToolBar* toolBar(const QLatin1String &toolBarId) const;
+    void recalcWordWrap(int visibleTextWidthForPrinting = -1);
+
+    bool canCut();
+    bool canCopy();
+
+    bool mInitCalled = false;
     /** the configuration object of the application */
     //KConfig *config;
 
     // QAction pointers to enable/disable actions
-    QAction* fileOpen;
-    QAction* fileSave;
-    QAction* fileSaveAs;
-    QAction* filePrint;
-    QAction* fileQuit;
-    QAction* fileReload;
-    QAction* editCut;
-    QAction* editCopy;
-    QAction* editPaste;
-    QAction* editSelectAll;
-    KToggleAction* viewToolBar;
+    QPointer<QAction> fileOpen;
+    QPointer<QAction> fileSave;
+    QPointer<QAction> fileSaveAs;
+    QPointer<QAction> filePrint;
+    QPointer<QAction> fileQuit;
+    QPointer<QAction> fileReload;
+    QPointer<QAction> editCut;
+    QPointer<QAction> editCopy;
+    QPointer<QAction> editPaste;
+    QPointer<QAction> editSelectAll;
+    KToggleAction* viewToolBar = nullptr;
     KToggleAction* viewStatusBar;
 
     ////////////////////////////////////////////////////////////////////////
     // Special KDiff3 specific stuff starts here
-    QAction* editFind;
-    QAction* editFindNext;
+    QPointer<QAction> editFind;
+    QPointer<QAction> editFindNext;
 
-    QAction* goCurrent;
-    QAction* goTop;
-    QAction* goBottom;
-    QAction* goPrevUnsolvedConflict;
-    QAction* goNextUnsolvedConflict;
-    QAction* goPrevConflict;
-    QAction* goNextConflict;
-    QAction* goPrevDelta;
-    QAction* goNextDelta;
+    QPointer<QAction> mGoCurrent;
+    QPointer<QAction> mGoTop;
+    QPointer<QAction> mGoBottom;
+    QPointer<QAction> mGoPrevUnsolvedConflict;
+    QPointer<QAction> mGoNextUnsolvedConflict;
+    QPointer<QAction> mGoPrevConflict;
+    QPointer<QAction> mGoNextConflict;
+    QPointer<QAction> mGoPrevDelta;
+    QPointer<QAction> mGoNextDelta;
     KToggleAction* chooseA;
     KToggleAction* chooseB;
     KToggleAction* chooseC;
     KToggleAction* autoAdvance;
     KToggleAction* wordWrap;
-    QAction* splitDiff;
-    QAction* joinDiffs;
-    QAction* addManualDiffHelp;
-    QAction* clearManualDiffHelpList;
+    QPointer<QAction> splitDiff;
+    QPointer<QAction> joinDiffs;
+    QPointer<QAction> addManualDiffHelp;
+    QPointer<QAction> clearManualDiffHelpList;
     KToggleAction* showWhiteSpaceCharacters;
     KToggleAction* showWhiteSpace;
     KToggleAction* showLineNumbers;
-    QAction* autoSolve;
-    QAction* unsolve;
-    QAction* mergeHistory;
-    QAction* mergeRegExp;
+    QPointer<QAction> mAutoSolve;
+    QPointer<QAction> mUnsolve;
+    QPointer<QAction> mMergeHistory;
+    QPointer<QAction> mergeRegExp;
     KToggleAction* showWindowA;
     KToggleAction* showWindowB;
     KToggleAction* showWindowC;
-    QAction* winFocusNext;
-    QAction* winFocusPrev;
-    QAction* winToggleSplitOrientation;
+    QPointer<QAction> winFocusNext;
+    QPointer<QAction> winFocusPrev;
+    QPointer<QAction> winToggleSplitOrientation;
     KToggleAction* dirShowBoth;
-    QAction* dirViewToggle;
+    QPointer<QAction> dirViewToggle;
     KToggleAction* overviewModeNormal;
     KToggleAction* overviewModeAB;
     KToggleAction* overviewModeAC;
@@ -315,39 +399,37 @@ class KDiff3App : public QSplitter
 
     QMenu* m_pMergeEditorPopupMenu;
 
-    QSplitter* m_pMainSplitter;
-    QWidget* m_pMainWidget;
-    QWidget* m_pMergeWindowFrame;
+    QSplitter* m_pMainSplitter = nullptr;
+    QWidget* m_pMainWidget = new QWidget();// Contains vertical splitter and horiz scrollbar
+    QWidget* m_pMergeWindowFrame = nullptr;
     ReversibleScrollBar* m_pHScrollBar = nullptr;
-    QScrollBar* m_pDiffVScrollBar = nullptr;
-    QScrollBar* m_pMergeVScrollBar;
 
-    DiffTextWindow* m_pDiffTextWindow1;
-    DiffTextWindow* m_pDiffTextWindow2;
-    DiffTextWindow* m_pDiffTextWindow3;
-    DiffTextWindowFrame* m_pDiffTextWindowFrame1;
-    DiffTextWindowFrame* m_pDiffTextWindowFrame2;
-    DiffTextWindowFrame* m_pDiffTextWindowFrame3;
-    QSplitter* m_pDiffWindowSplitter;
+    DiffTextWindow* m_pDiffTextWindow1 = nullptr;
+    DiffTextWindow* m_pDiffTextWindow2 = nullptr;
+    DiffTextWindow* m_pDiffTextWindow3 = nullptr;
+    DiffTextWindowFrame* m_pDiffTextWindowFrame1 = nullptr;
+    DiffTextWindowFrame* m_pDiffTextWindowFrame2 = nullptr;
+    DiffTextWindowFrame* m_pDiffTextWindowFrame3 = nullptr;
+    QSplitter* m_pDiffWindowSplitter = nullptr;
 
-    MergeResultWindow* m_pMergeResultWindow;
+    MergeResultWindow* m_pMergeResultWindow = nullptr;
     WindowTitleWidget* m_pMergeResultWindowTitle;
-    bool m_bTripleDiff;
+    static bool m_bTripleDiff;
 
-    QSplitter* m_pDirectoryMergeSplitter;
-    DirectoryMergeWindow* m_pDirectoryMergeWindow;
+    QSplitter* m_pDirectoryMergeSplitter = nullptr;
+    DirectoryMergeWindow* m_pDirectoryMergeWindow = nullptr;
     DirectoryMergeInfo* m_pDirectoryMergeInfo;
     bool m_bDirCompare = false;
 
-    Overview* m_pOverview;
+    Overview* m_pOverview = nullptr;
 
-    QWidget* m_pCornerWidget;
+    QWidget* m_pCornerWidget = nullptr;
 
-    TotalDiffStatus m_totalDiffStatus;
+    TotalDiffStatus *m_totalDiffStatus = new TotalDiffStatus();
 
-    SourceData m_sd1;
-    SourceData m_sd2;
-    SourceData m_sd3;
+    QSharedPointer<SourceData> m_sd1 = QSharedPointer<SourceData>::create();
+    QSharedPointer<SourceData> m_sd2 = QSharedPointer<SourceData>::create();
+    QSharedPointer<SourceData> m_sd3 = QSharedPointer<SourceData>::create();
 
     QSharedPointer<class DirectoryInfo> m_dirinfo;
 
@@ -358,7 +440,7 @@ class KDiff3App : public QSplitter
     DiffList m_diffList23;
     DiffList m_diffList13;
 
-    DiffBufferInfo m_diffBufferInfo;
+    QSharedPointer<DiffBufferInfo> m_diffBufferInfo = QSharedPointer<DiffBufferInfo>::create();
     Diff3LineList m_diff3LineList;
     Diff3LineVector m_diff3LineVector;
     //ManualDiffHelpDialog* m_pManualDiffHelpDialog;
@@ -366,44 +448,30 @@ class KDiff3App : public QSplitter
 
     int m_neededLines;
     int m_DTWHeight;
-    bool m_bOutputModified;
-    bool m_bFileSaved;
-    bool m_bTimerBlock; // Synchronization
+    bool m_bOutputModified = false;
+    bool m_bFileSaved = false;
+    bool m_bTimerBlock = false; // Synchronization
 
-    OptionDialog* m_pOptionDialog;
-    Options* m_pOptions;
-    FindDialog* m_pFindDialog;
+    OptionDialog* m_pOptionDialog = nullptr;
+    QSharedPointer<Options> m_pOptions = nullptr;
+    FindDialog* m_pFindDialog = nullptr;
 
-    void mainInit(TotalDiffStatus* pTotalDiffStatus = nullptr, bool bLoadFiles = true, bool bUseCurrentEncoding = false);
-    bool m_bFinishMainInit;
-    bool m_bLoadFiles;
+    bool m_bFinishMainInit = false;
+    bool m_bLoadFiles = false;
 
-    void mainWindowEnable(bool bEnable);
-    void wheelEvent(QWheelEvent* pWheelEvent) override;
-    void keyPressEvent(QKeyEvent* event) override;
-    bool eventFilter(QObject* o, QEvent* e) override;
-    void resizeEvent(QResizeEvent*) override;
-
-    bool improveFilenames(bool bCreateNewInstance);
-
-    bool canContinue();
-
-    void choose(e_SrcSelector choice);
-
-    KActionCollection* actionCollection();
-    QStatusBar* statusBar();
-    KToolBar* toolBar(QLatin1String);
-    KDiff3Part* m_pKDiff3Part;
-    KParts::MainWindow* m_pKDiff3Shell;
+    KDiff3Part* m_pKDiff3Part = nullptr;
+    KParts::MainWindow* m_pKDiff3Shell = nullptr;
     bool m_bAutoFlag = false;
     bool m_bAutoMode = false;
-    void recalcWordWrap(int visibleTextWidthForPrinting = -1);
-    bool m_bRecalcWordWrapPosted;
-    void setHScrollBarRange();
+    bool m_bRecalcWordWrapPosted = false;
 
-    int m_iCumulativeWheelDelta;
     int m_firstD3LIdx;                 // only needed during recalcWordWrap
     QPointer<QEventLoop> m_pEventLoopForPrinting;
+
+    /*
+      This list exists solely to auto disconnect boost signals.
+    */
+    std::list<boost::signals2::scoped_connection> connections;
 };
 
 #endif // KDIFF3_H
