@@ -1,22 +1,11 @@
-/***************************************************************************
- *   Copyright (C) 2003-2007 by Joachim Eibl <joachim.eibl at gmx.de>      *
- *   Copyright (C) 2018 Michael Reeves reeves.87@gmail.com                 *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
- ***************************************************************************/
+/*
+ * KDiff3 - Text Diff And Merge Tool
+ *
+ * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
+ * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
+ * SPDX-License-Identifier: GPL-2.0-or-later
+*/
+//clazy:excludeall=connect-non-signal,incorrect-emit
 
 #include "kdiff3_shell.h"
 #include "kdiff3.h"
@@ -44,14 +33,14 @@ KDiff3Shell::KDiff3Shell(bool bCompleteInit)
     statusBar()->show();
 
     /*const QVector<KPluginMetaData> plugin_offers = KPluginLoader::findPlugins( "kf5/kdiff3part" );
-    foreach( const KPluginMetaData & service, plugin_offers ) {
+    for( const KPluginMetaData & service: plugin_offers ) {
         KPluginFactory *factory = KPluginLoader( service.fileName() ).factory();
         m_part = factory->create<KDiff3Part>( this, QVariantList() << QVariant( QLatin1String( "KDiff3Part" ) ) );
         if( m_part )
             break;
     }*/
 
-    m_part = new KDiff3Part(this, this, QVariantList() << QVariant(QLatin1String("KDiff3Part")));
+    m_part = new KDiff3Part(this, this, {QVariant(QLatin1String("KDiff3Part"))});
     m_widget = qobject_cast<KDiff3App*>(m_part->widget());
 
     if(m_part)
@@ -65,7 +54,7 @@ KDiff3Shell::KDiff3Shell(bool bCompleteInit)
 
         if(bCompleteInit)
             m_widget->completeInit(QString());
-        connect(m_widget, &KDiff3App::createNewInstance, this, &KDiff3Shell::slotNewInstance);
+        chk_connect(m_widget, &KDiff3App::createNewInstance, this, &KDiff3Shell::slotNewInstance);
     }
     else
     {
@@ -91,14 +80,12 @@ KDiff3Shell::KDiff3Shell(bool bCompleteInit)
     m_bUnderConstruction = false;
 }
 
-KDiff3Shell::~KDiff3Shell()
-{
-}
+KDiff3Shell::~KDiff3Shell() = default;
 
 bool KDiff3Shell::queryClose()
 {
-    if(m_part)
-        return ((KDiff3App*)m_part->widget())->queryClose();
+    if(m_widget)
+        return m_widget->queryClose();
     else
         return true;
 }
@@ -113,8 +100,8 @@ void KDiff3Shell::closeEvent(QCloseEvent* e)
     if(queryClose())
     {
         e->accept();
-        bool bFileSaved = ((KDiff3App*)m_part->widget())->isFileSaved();
-        bool bDirCompare = ((KDiff3App*)m_part->widget())->isDirComparison();
+        bool bFileSaved = m_widget->isFileSaved();
+        bool bDirCompare = m_widget->isDirComparison();
         QApplication::exit(bFileSaved || bDirCompare ? 0 : 1);
     }
     else
@@ -153,7 +140,7 @@ void KDiff3Shell::optionsConfigureToolbars()
 
     // use the standard toolbar editor
     KEditToolBar dlg(factory());
-    connect(&dlg, &KEditToolBar::newToolBarConfig, this, &KDiff3Shell::applyNewToolbarConfig);
+    chk_connect(&dlg, &KEditToolBar::newToolBarConfig, this, &KDiff3Shell::applyNewToolbarConfig);
     dlg.exec();
 }
 
@@ -166,7 +153,7 @@ void KDiff3Shell::applyNewToolbarConfig()
 void KDiff3Shell::slotNewInstance(const QString& fn1, const QString& fn2, const QString& fn3)
 {
     static KDiff3Shell* pKDiff3Shell = new KDiff3Shell(false);
-    ((KDiff3App*)pKDiff3Shell->m_part->widget())->completeInit(fn1, fn2, fn3);
+    pKDiff3Shell->m_widget->completeInit(fn1, fn2, fn3);
 }
 
 //#include "kdiff3_shell.moc"
